package org.implab.gradle.containers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.gradle.api.logging.Logger;

public interface ExecuteMixin {

    Logger getLogger();

    Optional<File> getWorkingDir();

    default void Execute() throws Exception {
        final Logger log = getLogger();

        List<String> command = new ArrayList<>();

        preparingCommand(command);

        ProcessBuilder builder = new ProcessBuilder(command);
        getWorkingDir().ifPresent(workingDir -> builder.directory(workingDir));

        log.info("Starting: {}", builder.command());

        Process p = builder.start();

        processStarted(p);

        if (log.isErrorEnabled()) {
            Utils.redirectIO(p.getErrorStream(), log::error);
        }

        int code = p.waitFor();
        if (code != 0)
            throw new Exception("Process exited with the error: " + code);
    }

    void preparingCommand(List<String> command);


    /**
     * Invoked after the process is started, can be used to handle the process
     * output.
     * 
     * Default implementation redirects output to the logger as INFO messages.
     * 
     * @param p The current process.
     */

    default void processStarted(Process p) {
        final Logger log = getLogger();
        Utils.redirectIO(p.getInputStream(), log::info);
    }
}
